#
# Autogenerated by Thrift
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
package Net::GenCassandra::Constants;
require 5.6.0;
use strict;
use warnings;
use Net::GenThrift::Thrift;


use constant VERSION => "2.2.0";

1;
